
import java.util.Vector;               // Classe di una libreria di Java (package)


// Immagine:

int IMG_SIZE = 800;                    // dimensione dei lati (pixel)


// Orologio:

int SNAPSHOT = 50;                     // "quanto" di tempo


// Elenco delle lucciole (sequenza di tipo Vector):

Vector<Firefly> fireflies = new Vector<Firefly>();


// Avvio del programma

void setup() {

  size( 800, 800 );                    // IMG_SIZE x IMG_SIZE
  
  // Distribuzione reticolare delle lucciole (matrice temporanea GRID x GRID):

  int GRID = 5;
  Firefly[][] firefly = new Firefly[ GRID ][ GRID ];
  
  for ( int i=0; i<GRID; i++ ) {
    for ( int j=0; j<GRID; j++ ) {
    
      // Creazione di una nuova lucciola in una determinata posizione:
      
      int x = IMG_SIZE*(i+1)/(GRID+1);
      int y = IMG_SIZE*(j+1)/(GRID+1);
      
      firefly[i][j] = new Firefly( x, y );
    
      // Identificazione reciproca delle lucciole vicine (create in precedenza):
      
      if ( (i > 0) && (j > 0) ) {
        firefly[i][j].neighbor( firefly[i-1][j-1] );
        firefly[i-1][j-1].neighbor( firefly[i][j] );
      }
      if ( i > 0 ) {
        firefly[i][j].neighbor( firefly[i-1][j] );
        firefly[i-1][j].neighbor( firefly[i][j] );
      }
      if ( (i > 0) && (j+1 < GRID) ) {
        firefly[i][j].neighbor( firefly[i-1][j+1] );
        firefly[i-1][j+1].neighbor( firefly[i][j] );
      }
      if ( j > 0 ) {
        firefly[i][j].neighbor( firefly[i][j-1] );
        firefly[i][j-1].neighbor( firefly[i][j] );
      }
  }}
  
  // Avvio di un thread autonomo per ciascuna delle lucciole
  
  for ( int i=0; i<GRID; i++ ) {
    for ( int j=0; j<GRID; j++ ) {
    
      fireflies.add( firefly[i][j] );  // nuova lucciola registrata nell'elenco
      thread( "activateFirefly" );     // lampeggio come processo autonomo (thread)
      
      delay( SNAPSHOT );               // ritardo per assicurare l'avvio del processo
  }}
}


// Visualizzazione

void draw() {

  setDark();                           // colorazione dello sfondo (blu notte)
  rect( 0, 0, IMG_SIZE, IMG_SIZE );
  
  for ( Firefly ffly : fireflies ) {   // per tutte le lucciole dell'elenco
  
    ffly.showFirefly();                // visualizzazione della lucciola
  }
}


// Processo autonomo (thread) eseguito in parallelo per ciascuna lucciola:
// l'ultima lucciola registrata nell'elenco e' di volta in volta diversa
// e si avvieranno tanti processi autonomi quante sono le lucciole

void activateFirefly() {

  ( fireflies.lastElement() ).flicker();
}


// Procedure di supporto

void setDark() {                       // colore blu notte

  stroke( 0, 0, 64 );
  fill( 0, 0, 64 );
}

void setBright() {                     // colore della luce prodotta
                                       // dall'addome dell'insetto
  stroke( 196, 255, 255 );
  fill( 196, 255, 255 );
}

void randomDelay() {                   // ritardo generato casualmente

  int d = (int) ( 100*Math.random() );
  delay( d * SNAPSHOT );               // < 5 sec: intervallo che copre 2-3 cicli
}


// Modello di ciascuna lucciola

class Firefly {

  int x, y;                            // posizione della lucciola
  boolean bright;                      // emmissione di luce?
  Vector<Firefly> neighbors;           // lucciole vicine
  
  // --------------------------------- // ulteriori informazioni di stato relative
                                       // all'accelerazione dell'orologio interno:
  // --------------------------------- // da definire!
  
  Firefly( int px, int py ) {          // creazione di una nuova lucciola
  
    x = px;                            // in una data posizione
    y = py;
    bright = false;                    // non luminescente e "vicinato" da popolare
    neighbors = new Vector<Firefly>();
  }
  
  void flicker() {                     // ciclo di lampeggiamento
  
    // randomDelay();                  // inizio del ciclo asincrono
    
    while ( true ) {                   // ripetizione senza fine
    
      bright = true;                   // attivazione luminescenza
      stimulateNeighbors();            // stimolazione delle lucciole vicine
      
      delay( 500 );                    // durata della luminescenza (0.5 sec)
      
      // ----------------------------- // sezione da completare opportunamente
      
                                       // periodo insensibile agli stimoli luminosi
      
      delay( 2000 );                   // periodo indicativo di assenza di luminescenza
                                       // inserito in questo modo solo provvisoriamente,
                                       // ma da rielaborare in relazione al contesto
      
                                       // periodo sensibile agli stimoli luminosi
      
                                       // breve intervallo di tempo suscettibile a
                                       // essere ridotto in relazione agli stimoli
      
      // ----------------------------- // fine sezione da completare
    }
  }
  
  void neighbor( Firefly ffly ) {      // identificazione di una lucciola vicina
  
    neighbors.add( ffly );             // registrazione nell'elenco
  }
  
  void lightStimulus() {               // stimolo luminoso da una lucciola vicina
  
    // ------------------------------- // metodo da definire opportunamente
    
                                       // in relazione a come opera flicker
    
    // ------------------------------- // fine metodo da completare
  }
  
  void stimulateNeighbors() {          // stimolazione delle lucciole vicine
  
    for ( Firefly ffly : neighbors ) {
    
      ffly.lightStimulus();
    }
  }
  
  void showFirefly() {                 // visualizzazione della lucciola
  
    if ( bright ) {                    // la colorazione dipende dallo stato
      setBright();                     // di luminescenza
    } else {
      setDark();                       // o meno
    }
    ellipse( x, y, 10, 10 );
  }

}  // class Firefly
