
import java.util.Vector;


int SNAPSHOT = 1500;


void setup() {

  size( 600, 600 );
  
  roundTable = new RoundTable( 12 );
  
  thread( "josephus" );
}


RoundTable roundTable;


void josephus() {

  roundTable.haveDrinkTogether();
}


class RoundTable {

  Vector<Diner> diners;

  RoundTable( int n ) {
  
    diners = new Vector<Diner>();
    
    double phi = 3 * Math.PI / 2;
    double delta = 2 * Math.PI / n;
    int x = (int) ( 300 + 200 * Math.cos(phi) );
    int y = (int) ( 300 + 200 * Math.sin(phi) );
    
    Diner first = new Diner( 1, x, y );
    diners.add( first );
    Diner diner = first;
   
    for ( int k = 2;  k <= n;  k = k+1 ) {
    
      phi = phi + delta;
      x = (int) ( 300 + 200 * Math.cos(phi) );
      y = (int) ( 300 + 200 * Math.sin(phi) );
      
      Diner next = new Diner( k, x, y );
      diners.add( next );
      diner.identifyLeftNeighbor( next );
      diner = next;
    }
    diner.identifyLeftNeighbor( first );
  }
  
  void haveDrinkTogether() {
  
    Diner diner = diners.firstElement();
    diner.graspJar();
    
    delay( 2*SNAPSHOT );
    
    for ( int n = diners.size();  n > 2;  n = n-1 ) {
    
      diner = diner.passJar();
      delay( SNAPSHOT );
      
      diner.pourWine();
      delay( SNAPSHOT );
      
      diner = diner.passJar();
      delay( 10 * SNAPSHOT );
    }
  }
  
  
  void drawRoundTable() {
  
    for ( Diner diner : diners ) {
    
      diner.drawDiner();
    }
  }

}  // class RoundTable


class Diner {

  int myNum;
  Diner onMyLeft;
  boolean haveJar;
  boolean canDrink;
  
  int myX;
  int myY;
  
  Diner( int n, int x, int y ) {
  
    myNum = n;
    onMyLeft = null;
    haveJar = false;
    canDrink = false;
    
    myX = x;
    myY = y;
  }
  
  int number() {
  
    return myNum;
  }
  
  void identifyLeftNeighbor( Diner d ) {
  
    onMyLeft = d;
  }
  
  void getDrink() {
  
    canDrink = true;
  }
  
  void graspJar() {
  
    haveJar = true;
  }
  
  void pourWine() {
  
    if ( haveJar ) {
      onMyLeft.getDrink();
      identifyLeftNeighbor( onMyLeft.leftNeighbor() );
    }
  }
  
  Diner passJar() {
  
    if ( haveJar ) {
      onMyLeft.graspJar();
      haveJar = false;
    }
    return onMyLeft;
  }
  
  Diner leftNeighbor() {
  
    return onMyLeft;
  }
  
  
  void drawDiner() {
  
    if ( canDrink ) {
      stroke( 200, 200, 0 );
      fill( 200, 200, 100 );
    } else if ( haveJar ) {
      stroke( 255, 0, 0 );
      fill( 255, 0, 0 );
    } else {
      stroke( 0, 0, 255 );
      fill( 0, 0, 255 );
    }
    ellipse( myX, myY, 40, 40 );

    textSize( 26 );
    fill( 255, 255, 0 );
    if ( myNum < 10 ) {
      text( ""+myNum, myX-8, myY+9 );
    } else {
      text( ""+myNum, myX-15, myY+9 );
    }
  }

}  // class Diner


void drawBackground() {

  stroke( 200, 200, 200 );
  fill( 200, 200, 200 );
  rect( 0, 0, 600, 600 );
}


void draw() {

  drawBackground();
  roundTable.drawRoundTable();
}
