
// Immagine:

int IMG_SIZE = 600;                    // dimensione dei lati (pixel)

int X0 = IMG_SIZE / 2;                 // coordinate del centro
int Y0 = IMG_SIZE / 2;


// Elenco dei semi:

int N_SEEDS = 8;                       // 8, 16, 32, 64, 128, ... 1024
Seed[] seeds = new Seed[N_SEEDS];


// Avvio del programma

void setup() {

  // Area immagine:
  
  size( 600, 600 );                    // IMG_SIZE x IMG_SIZE
  
  // Generazione e sviluppo dei semi:
  
  // --------------------------------- // angolo di rotazione
  double phi = 0.125;
  // --------------------------------- // da regolare!
  
  double rotation = phi * 2*Math.PI;   // angolo di rotazione
  
  double theta = 0;                    // direzione del successivo primordio

  for ( int t=N_SEEDS-1; t>=0; t=t-1 ) {
  
    seeds[t] = new Seed( theta );      // creazione di un nuovo primordio
    theta = theta + rotation;          // direzione del successivo
    
    seeds[t].develop( t );             // disposizione dopo un intervallo di tempo t
  }
}


// Realizzazione dell'immagine

void draw() {

  stroke( 223, 191, 0 );               // colorazione dello sfondo
  fill( 223, 191, 0 );
  rect( 0, 0, IMG_SIZE, IMG_SIZE );
  
  for ( int k=0; k<N_SEEDS; k=k+1 ) {  // per tutti i semi dell'elenco...
  
    seeds[k].drawSeed();               // immagine del seme
  }
}


// Modello di ciascun seme

class Seed {

  double s = 7;                        // raggio dell'area del seme

  double theta;                        // direzione di sviluppo
  double r;                            // distanza dal centro dell'apice
  
  Seed( double th ) {                  // generazione di un nuovo seme
  
    theta = th;                        // orientato in direzione th
    r = s;                             // distanza iniziale dal centro dell'apice
  }
  
  void develop( int t ) {              // sviluppo radiale del seme
  
    for ( int i=0; i<t; i=i+1 ) {      // per t intervalli elementari di tempo
    
      // ----------------------------- // spostamento nell'intervallo di tempo
      double delta = 1.5; 
      // ----------------------------- // da accordare con il modello teorico!
      
      r = r + delta;
    }
  }
  
  void drawSeed() {
    
    // Posizione del (centro del) seme nell'immagine:
  
    int x = X0 + (int) ( r * Math.cos(theta) );
    int y = Y0 + (int) ( r * Math.sin(theta) );
    
    stroke( 128, 144, 0 );             // colorazione del seme
    fill( 128, 144, 0 );
    ellipse( x, y, 10, 10 );           // raggio seme (5) < raggio area relativa (s)
  }

}    // class Seed
